@@ -24,6 +24,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#undef WNCK_DISABLE_DEPRECATED
+
 #include <config.h>
 
 #include <math.h>
@@ -32,7 +34,6 @@
 #include "pager.h"
 #include "workspace.h"
 #include "window.h"
-#include "window-action-menu.h"
 #include "xutils.h"
 #include "pager-accessible-factory.h"
 #include "workspace-accessible-factory.h"
@@ -175,6 +176,8 @@ static void wnck_pager_connect_screen    (WnckPager  *pager);
 static void wnck_pager_connect_window    (WnckPager  *pager,
                                           WnckWindow *window);
 static void wnck_pager_disconnect_screen (WnckPager  *pager);
+static void wnck_pager_disconnect_window (WnckPager  *pager,
+                                          WnckWindow *window);
 
 static gboolean wnck_pager_set_layout_hint (WnckPager  *pager);
 
@@ -290,6 +293,9 @@ _wnck_pager_set_screen (WnckPager *pager)
 {
   GdkScreen *gdkscreen;
 
+  if (!gtk_widget_has_screen (GTK_WIDGET (pager)))
+    return;
+
   gdkscreen = gtk_widget_get_screen (GTK_WIDGET (pager));
   pager->priv->screen = wnck_screen_get (gdk_screen_get_number (gdkscreen));
 
@@ -1949,7 +1955,9 @@ wnck_pager_set_layout_hint (WnckPager *pager)
   /* if we're not realized, we don't know about our screen yet */
   if (pager->priv->screen == NULL)
     _wnck_pager_set_screen (pager);
-  g_assert (pager->priv->screen != NULL);
+  /* can still happen if the pager was not added to a widget hierarchy */
+  if (pager->priv->screen == NULL)
+    return FALSE;
 
   /* The visual representation of the pager doesn't
    * correspond to the layout of the workspaces
@@ -1999,6 +2007,9 @@ wnck_pager_set_layout_hint (WnckPager *pager)
  * For example, if the layout contains one row, but the orientation of the
  * layout is vertical, the #WnckPager will display a column of #WnckWorkspace.
  *
+ * If @pager has not been added to a widget hierarchy, the call will fail
+ * because @pager can't know the screen on which to modify the orientation.
+ *
  * Return value: %TRUE if the layout of #WnckWorkspace has been successfully
  * changed or did not need to be changed, %FALSE otherwise.
  */
@@ -2043,6 +2054,9 @@ wnck_pager_set_orientation (WnckPager     *pager,
  * set this property of a #WnckScreen at a time, setting the layout is not
  * guaranteed to work. 
  *
+ * If @pager has not been added to a widget hierarchy, the call will fail
+ * because @pager can't know the screen on which to modify the layout.
+ *
  * Return value: %TRUE if the layout of #WnckWorkspace has been successfully
  * changed or did not need to be changed, %FALSE otherwise.
  */
@@ -2405,27 +2419,28 @@ static void
 wnck_pager_connect_window (WnckPager  *pager,
                            WnckWindow *window)
 {
-  g_signal_connect_object (G_OBJECT (window), "name_changed",
-                           G_CALLBACK (window_name_changed_callback),
-                           pager, 0);
-  g_signal_connect_object (G_OBJECT (window), "state_changed",
-                           G_CALLBACK (window_state_changed_callback),
-                           pager, 0);
-  g_signal_connect_object (G_OBJECT (window), "workspace_changed",
-                           G_CALLBACK (window_workspace_changed_callback),
-                           pager, 0);
-  g_signal_connect_object (G_OBJECT (window), "icon_changed",
-                           G_CALLBACK (window_icon_changed_callback),
-                           pager, 0);
-  g_signal_connect_object (G_OBJECT (window), "geometry_changed",
-                           G_CALLBACK (window_geometry_changed_callback),
-                           pager, 0);
+  g_signal_connect (G_OBJECT (window), "name_changed",
+                    G_CALLBACK (window_name_changed_callback),
+                    pager);
+  g_signal_connect (G_OBJECT (window), "state_changed",
+                    G_CALLBACK (window_state_changed_callback),
+                    pager);
+  g_signal_connect (G_OBJECT (window), "workspace_changed",
+                    G_CALLBACK (window_workspace_changed_callback),
+                    pager);
+  g_signal_connect (G_OBJECT (window), "icon_changed",
+                    G_CALLBACK (window_icon_changed_callback),
+                    pager);
+  g_signal_connect (G_OBJECT (window), "geometry_changed",
+                    G_CALLBACK (window_geometry_changed_callback),
+                    pager);
 }
 
 static void
 wnck_pager_disconnect_screen (WnckPager  *pager)
 {
   int i;
+  GList *tmp;
 
   if (pager->priv->screen == NULL)
     return;
@@ -2448,6 +2463,32 @@ wnck_pager_disconnect_screen (WnckPager  *pager)
       space = wnck_screen_get_workspace (pager->priv->screen, i);
       g_signal_handlers_disconnect_by_func (space, G_CALLBACK (workspace_name_changed_callback), pager);
     }
+ 
+  for (tmp = wnck_screen_get_windows (pager->priv->screen); tmp; tmp = tmp->next)
+    {
+      wnck_pager_disconnect_window (pager, WNCK_WINDOW (tmp->data));
+    }
+}
+
+static void
+wnck_pager_disconnect_window (WnckPager  *pager,
+                              WnckWindow *window)
+{
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window),
+                                        G_CALLBACK (window_name_changed_callback),
+                                        pager);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window),
+                                        G_CALLBACK (window_state_changed_callback),
+                                        pager);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window),
+                                        G_CALLBACK (window_workspace_changed_callback),
+                                        pager);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window),
+                                        G_CALLBACK (window_icon_changed_callback),
+                                        pager);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window),
+                                        G_CALLBACK (window_geometry_changed_callback),
+                                        pager);
 }
 
 static void
